@@ -53,16 +53,18 @@ RZ_API bool rz_analysis_op_fini(RzAnalysisOp *op) {
 	rz_analysis_switch_op_free(op->switch_op);
 	op->switch_op = NULL;
 	RZ_FREE(op->mnemonic);
+	rz_il_op_effect_free(op->il_op);
+	op->il_op = NULL;
 	return true;
 }
 
-RZ_API void rz_analysis_op_free(void *_op) {
-	if (!_op) {
+RZ_API void rz_analysis_op_free(void *op) {
+	if (!op) {
 		return;
 	}
-	rz_analysis_op_fini(_op);
-	memset(_op, 0, sizeof(RzAnalysisOp));
-	free(_op);
+	rz_analysis_op_fini(op);
+	memset(op, 0, sizeof(RzAnalysisOp));
+	free(op);
 }
 
 static int defaultCycles(RzAnalysisOp *op) {
@@ -97,15 +99,15 @@ RZ_API int rz_analysis_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, con
 
 	int ret = RZ_MIN(2, len);
 	if (len > 0 && analysis->cur && analysis->cur->op) {
-		//use core binding to set asm.bits correctly based on the addr
-		//this is because of the hassle of arm/thumb
+		// use core binding to set asm.bits correctly based on the addr
+		// this is because of the hassle of arm/thumb
 		if (analysis && analysis->coreb.archbits) {
 			analysis->coreb.archbits(analysis->coreb.core, addr);
 		}
 		if (analysis->pcalign && addr % analysis->pcalign) {
 			op->type = RZ_ANALYSIS_OP_TYPE_ILL;
 			op->addr = addr;
-			// eprintf ("Unaligned instruction for %d bits at 0x%"PFMT64x"\n", analysis->bits, addr);
+			// RZ_LOG_DEBUG("Unaligned instruction for %d bits at 0x%"PFMT64x"\n", analysis->bits, addr);
 			op->size = 1;
 			return -1;
 		}
@@ -127,9 +129,7 @@ RZ_API int rz_analysis_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, con
 		}
 	}
 	if (!op->mnemonic && (mask & RZ_ANALYSIS_OP_MASK_DISASM)) {
-		if (analysis->verbose) {
-			eprintf("Warning: unhandled RZ_ANALYSIS_OP_MASK_DISASM in rz_analysis_op\n");
-		}
+		RZ_LOG_DEBUG("Warning: unhandled RZ_ANALYSIS_OP_MASK_DISASM in rz_analysis_op\n");
 	}
 	if (mask & RZ_ANALYSIS_OP_MASK_HINT) {
 		RzAnalysisHint *hint = rz_analysis_hint_get(analysis, addr);
@@ -177,7 +177,7 @@ RZ_API RzAnalysisOp *rz_analysis_op_copy(RzAnalysisOp *op) {
 RZ_API bool rz_analysis_op_nonlinear(int t) {
 	t &= RZ_ANALYSIS_OP_TYPE_MASK;
 	switch (t) {
-	//call
+	// call
 	case RZ_ANALYSIS_OP_TYPE_CALL:
 	case RZ_ANALYSIS_OP_TYPE_RCALL:
 	case RZ_ANALYSIS_OP_TYPE_ICALL:
@@ -233,7 +233,7 @@ static struct optype {
 	{ RZ_ANALYSIS_OP_TYPE_CJMP, "cjmp" },
 	{ RZ_ANALYSIS_OP_TYPE_MJMP, "mjmp" },
 	{ RZ_ANALYSIS_OP_TYPE_CMP, "cmp" },
-	{ RZ_ANALYSIS_OP_TYPE_IO, "cret" },
+	{ RZ_ANALYSIS_OP_TYPE_CRET, "cret" },
 	{ RZ_ANALYSIS_OP_TYPE_ILL, "ill" },
 	{ RZ_ANALYSIS_OP_TYPE_JMP, "jmp" },
 	{ RZ_ANALYSIS_OP_TYPE_LEA, "lea" },
@@ -267,15 +267,15 @@ static struct optype {
 	{ RZ_ANALYSIS_OP_TYPE_SWITCH, "switch" },
 	{ RZ_ANALYSIS_OP_TYPE_TRAP, "trap" },
 	{ RZ_ANALYSIS_OP_TYPE_UCALL, "ucall" },
-	{ RZ_ANALYSIS_OP_TYPE_RCALL, "rcall" }, // needs to be changed
-	{ RZ_ANALYSIS_OP_TYPE_ICALL, "ucall" }, // needs to be changed
-	{ RZ_ANALYSIS_OP_TYPE_IRCALL, "ucall" }, // needs to be changed
+	{ RZ_ANALYSIS_OP_TYPE_RCALL, "rcall" },
+	{ RZ_ANALYSIS_OP_TYPE_ICALL, "icall" },
+	{ RZ_ANALYSIS_OP_TYPE_IRCALL, "ircall" },
 	{ RZ_ANALYSIS_OP_TYPE_UCCALL, "uccall" },
 	{ RZ_ANALYSIS_OP_TYPE_UCJMP, "ucjmp" },
 	{ RZ_ANALYSIS_OP_TYPE_UJMP, "ujmp" },
-	{ RZ_ANALYSIS_OP_TYPE_RJMP, "rjmp" }, // needs to be changed
-	{ RZ_ANALYSIS_OP_TYPE_IJMP, "ujmp" }, // needs to be changed
-	{ RZ_ANALYSIS_OP_TYPE_IRJMP, "ujmp" }, // needs to be changed
+	{ RZ_ANALYSIS_OP_TYPE_RJMP, "rjmp" },
+	{ RZ_ANALYSIS_OP_TYPE_IJMP, "ijmp" },
+	{ RZ_ANALYSIS_OP_TYPE_IRJMP, "irjmp" },
 	{ RZ_ANALYSIS_OP_TYPE_UNK, "unk" },
 	{ RZ_ANALYSIS_OP_TYPE_UPUSH, "upush" },
 	{ RZ_ANALYSIS_OP_TYPE_RPUSH, "rpush" },
@@ -283,91 +283,45 @@ static struct optype {
 	{ RZ_ANALYSIS_OP_TYPE_XOR, "xor" },
 	{ RZ_ANALYSIS_OP_TYPE_CASE, "case" },
 	{ RZ_ANALYSIS_OP_TYPE_CPL, "cpl" },
-	{ RZ_ANALYSIS_OP_TYPE_CRYPTO, "crypto" },
-	{ 0, NULL }
+	{ RZ_ANALYSIS_OP_TYPE_CRYPTO, "crypto" }
 };
 
-RZ_API int rz_analysis_optype_from_string(const char *type) {
+/**
+ * Return the op type corresponding the given name
+ * \param  name       string, name of the optype
+ * \return type       int, id of the op type (one of \link _RzAnalysisOpType \endlink)
+ */
+RZ_API int rz_analysis_optype_from_string(RZ_NONNULL const char *name) {
+	rz_return_val_if_fail(name, -1);
 	int i;
-	for (i = 0; optypes[i].name; i++) {
-		if (!strcmp(optypes[i].name, type)) {
+	for (i = 0; RZ_ARRAY_SIZE(optypes); i++) {
+		if (!strcmp(optypes[i].name, name)) {
 			return optypes[i].type;
 		}
 	}
 	return -1;
 }
 
-RZ_API const char *rz_analysis_optype_to_string(int t) {
-	bool once = true;
-repeat:
-	// TODO: delete
-	switch (t) {
-	case RZ_ANALYSIS_OP_TYPE_IO: return "io";
-	case RZ_ANALYSIS_OP_TYPE_ACMP: return "acmp";
-	case RZ_ANALYSIS_OP_TYPE_ADD: return "add";
-	case RZ_ANALYSIS_OP_TYPE_SYNC: return "sync";
-	case RZ_ANALYSIS_OP_TYPE_AND: return "and";
-	case RZ_ANALYSIS_OP_TYPE_CALL: return "call";
-	case RZ_ANALYSIS_OP_TYPE_CCALL: return "ccall";
-	case RZ_ANALYSIS_OP_TYPE_CJMP: return "cjmp";
-	case RZ_ANALYSIS_OP_TYPE_MJMP: return "mjmp";
-	case RZ_ANALYSIS_OP_TYPE_CMP: return "cmp";
-	case RZ_ANALYSIS_OP_TYPE_CRET: return "cret";
-	case RZ_ANALYSIS_OP_TYPE_DIV: return "div";
-	case RZ_ANALYSIS_OP_TYPE_ILL: return "ill";
-	case RZ_ANALYSIS_OP_TYPE_JMP: return "jmp";
-	case RZ_ANALYSIS_OP_TYPE_LEA: return "lea";
-	case RZ_ANALYSIS_OP_TYPE_LEAVE: return "leave";
-	case RZ_ANALYSIS_OP_TYPE_LOAD: return "load";
-	case RZ_ANALYSIS_OP_TYPE_NEW: return "new";
-	case RZ_ANALYSIS_OP_TYPE_MOD: return "mod";
-	case RZ_ANALYSIS_OP_TYPE_CMOV: return "cmov";
-	case RZ_ANALYSIS_OP_TYPE_MOV: return "mov";
-	case RZ_ANALYSIS_OP_TYPE_CAST: return "cast";
-	case RZ_ANALYSIS_OP_TYPE_MUL: return "mul";
-	case RZ_ANALYSIS_OP_TYPE_NOP: return "nop";
-	case RZ_ANALYSIS_OP_TYPE_NOT: return "not";
-	case RZ_ANALYSIS_OP_TYPE_NULL: return "null";
-	case RZ_ANALYSIS_OP_TYPE_OR: return "or";
-	case RZ_ANALYSIS_OP_TYPE_POP: return "pop";
-	case RZ_ANALYSIS_OP_TYPE_PUSH: return "push";
-	case RZ_ANALYSIS_OP_TYPE_RPUSH: return "rpush";
-	case RZ_ANALYSIS_OP_TYPE_REP: return "rep";
-	case RZ_ANALYSIS_OP_TYPE_RET: return "ret";
-	case RZ_ANALYSIS_OP_TYPE_ROL: return "rol";
-	case RZ_ANALYSIS_OP_TYPE_ROR: return "ror";
-	case RZ_ANALYSIS_OP_TYPE_SAL: return "sal";
-	case RZ_ANALYSIS_OP_TYPE_SAR: return "sar";
-	case RZ_ANALYSIS_OP_TYPE_SHL: return "shl";
-	case RZ_ANALYSIS_OP_TYPE_SHR: return "shr";
-	case RZ_ANALYSIS_OP_TYPE_STORE: return "store";
-	case RZ_ANALYSIS_OP_TYPE_SUB: return "sub";
-	case RZ_ANALYSIS_OP_TYPE_SWI: return "swi";
-	case RZ_ANALYSIS_OP_TYPE_CSWI: return "cswi";
-	case RZ_ANALYSIS_OP_TYPE_SWITCH: return "switch";
-	case RZ_ANALYSIS_OP_TYPE_TRAP: return "trap";
-	case RZ_ANALYSIS_OP_TYPE_UCALL: return "ucall";
-	case RZ_ANALYSIS_OP_TYPE_RCALL: return "rcall"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_ICALL: return "ucall"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_IRCALL: return "ucall"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_UCCALL: return "uccall";
-	case RZ_ANALYSIS_OP_TYPE_UCJMP: return "ucjmp";
-	case RZ_ANALYSIS_OP_TYPE_UJMP: return "ujmp";
-	case RZ_ANALYSIS_OP_TYPE_RJMP: return "rjmp"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_IJMP: return "ujmp"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_IRJMP: return "ujmp"; // needs to be changed
-	case RZ_ANALYSIS_OP_TYPE_UNK: return "unk";
-	case RZ_ANALYSIS_OP_TYPE_UPUSH: return "upush";
-	case RZ_ANALYSIS_OP_TYPE_XCHG: return "xchg";
-	case RZ_ANALYSIS_OP_TYPE_XOR: return "xor";
-	case RZ_ANALYSIS_OP_TYPE_CASE: return "case";
-	case RZ_ANALYSIS_OP_TYPE_CPL: return "cpl";
-	case RZ_ANALYSIS_OP_TYPE_CRYPTO: return "crypto";
+/**
+ * Return the name of the given op type
+ * \param  type       int, id of the op type (one of \link _RzAnalysisOpType \endlink)
+ * \return name       string, string, name of the optype
+ */
+RZ_API const char *rz_analysis_optype_to_string(int type) {
+	int i;
+
+	for (i = 0; i < RZ_ARRAY_SIZE(optypes); i++) {
+		if (optypes[i].type == type) {
+			return optypes[i].name;
+		}
 	}
-	if (once) {
-		once = false;
-		t &= RZ_ANALYSIS_OP_TYPE_MASK; // ignore the modifier bits... we don't want this!
-		goto repeat;
+
+	type &= RZ_ANALYSIS_OP_TYPE_MASK;
+
+	for (i = 0; i < RZ_ARRAY_SIZE(optypes); i++) {
+		if (optypes[i].type == type) {
+			return optypes[i].name;
+		}
 	}
 	return "undefined";
 }
@@ -547,7 +501,7 @@ RZ_API char *rz_analysis_op_to_string(RzAnalysis *analysis, RzAnalysisOp *op) {
 	case RZ_ANALYSIS_OP_TYPE_ROR:
 	case RZ_ANALYSIS_OP_TYPE_SWITCH:
 	case RZ_ANALYSIS_OP_TYPE_CASE:
-		eprintf("Command not implemented.\n");
+		RZ_LOG_DEBUG("Command not implemented.\n");
 		free(r0);
 		free(a0);
 		free(a1);
@@ -582,45 +536,49 @@ RZ_API const char *rz_analysis_stackop_tostring(int s) {
 	return "unk";
 }
 
-RZ_API const char *rz_analysis_op_family_to_string(int n) {
-	switch (n) {
-	case RZ_ANALYSIS_OP_FAMILY_UNKNOWN: return "unk";
-	case RZ_ANALYSIS_OP_FAMILY_CPU: return "cpu";
-	case RZ_ANALYSIS_OP_FAMILY_SECURITY: return "sec";
-	case RZ_ANALYSIS_OP_FAMILY_FPU: return "fpu";
-	case RZ_ANALYSIS_OP_FAMILY_MMX: return "mmx";
-	case RZ_ANALYSIS_OP_FAMILY_SSE: return "sse";
-	case RZ_ANALYSIS_OP_FAMILY_PRIV: return "priv";
-	case RZ_ANALYSIS_OP_FAMILY_THREAD: return "thrd";
-	case RZ_ANALYSIS_OP_FAMILY_CRYPTO: return "crpt";
-	case RZ_ANALYSIS_OP_FAMILY_IO: return "io";
-	case RZ_ANALYSIS_OP_FAMILY_VIRT: return "virt";
+static const struct {
+	int id;
+	const char *name;
+} op_families[] = {
+	{ RZ_ANALYSIS_OP_FAMILY_CPU, "cpu" },
+	{ RZ_ANALYSIS_OP_FAMILY_FPU, "fpu" },
+	{ RZ_ANALYSIS_OP_FAMILY_MMX, "mmx" },
+	{ RZ_ANALYSIS_OP_FAMILY_SSE, "sse" },
+	{ RZ_ANALYSIS_OP_FAMILY_PRIV, "priv" },
+	{ RZ_ANALYSIS_OP_FAMILY_VIRT, "virt" },
+	{ RZ_ANALYSIS_OP_FAMILY_CRYPTO, "crpt" },
+	{ RZ_ANALYSIS_OP_FAMILY_IO, "io" },
+	{ RZ_ANALYSIS_OP_FAMILY_SECURITY, "sec" },
+	{ RZ_ANALYSIS_OP_FAMILY_THREAD, "thread" },
+};
+
+/**
+ * Return the name of the given op family
+ * \param  id       int, id of the operation family (one of \link RzAnalysisOpFamily \endlink)
+ * \return name     string, name of the op family
+ */
+RZ_API const char *rz_analysis_op_family_to_string(int id) {
+	int i;
+
+	for (i = 0; i < RZ_ARRAY_SIZE(op_families); i++) {
+		if (op_families[i].id == id) {
+			return op_families[i].name;
+		}
 	}
 	return NULL;
 }
 
-RZ_API int rz_analysis_op_family_from_string(const char *f) {
-	struct op_family {
-		const char *name;
-		int id;
-	};
-	static const struct op_family of[] = {
-		{ "cpu", RZ_ANALYSIS_OP_FAMILY_CPU },
-		{ "fpu", RZ_ANALYSIS_OP_FAMILY_FPU },
-		{ "mmx", RZ_ANALYSIS_OP_FAMILY_MMX },
-		{ "sse", RZ_ANALYSIS_OP_FAMILY_SSE },
-		{ "priv", RZ_ANALYSIS_OP_FAMILY_PRIV },
-		{ "virt", RZ_ANALYSIS_OP_FAMILY_VIRT },
-		{ "crpt", RZ_ANALYSIS_OP_FAMILY_CRYPTO },
-		{ "io", RZ_ANALYSIS_OP_FAMILY_IO },
-		{ "sec", RZ_ANALYSIS_OP_FAMILY_SECURITY },
-		{ "thread", RZ_ANALYSIS_OP_FAMILY_THREAD },
-	};
-
+/**
+ * Return the op family id given its name
+ * \param  name     string, name of the op family
+ * \return id       int, id of the operation family (one of \link RzAnalysisOpFamily \endlink)
+ */
+RZ_API int rz_analysis_op_family_from_string(RZ_NONNULL const char *name) {
 	int i;
-	for (i = 0; i < sizeof(of) / sizeof(of[0]); i++) {
-		if (!strcmp(f, of[i].name)) {
-			return of[i].id;
+	rz_return_val_if_fail(name, RZ_ANALYSIS_OP_FAMILY_UNKNOWN);
+	for (i = 0; i < RZ_ARRAY_SIZE(op_families); i++) {
+		if (!strcmp(name, op_families[i].name)) {
+			return op_families[i].id;
 		}
 	}
 	return RZ_ANALYSIS_OP_FAMILY_UNKNOWN;
