@@ -329,7 +329,7 @@ static ut32 cb(ArmOp *op) {
 	} else {
 		return UT32_MAX;
 	}
-	//printf ("%s %d, %llu\n", op->mnemonic, op->operands[0].reg, op->operands[1].immediate);
+	// printf ("%s %d, %llu\n", op->mnemonic, op->operands[0].reg, op->operands[1].immediate);
 	ut32 imm = op->operands[1].immediate;
 	data = k | encode1reg(op) | ((imm & 0x1c) << 27) | ((imm & 0x1fe0) << 11);
 	data = data | ((imm & 0x1fe000) >> 5);
@@ -637,6 +637,52 @@ static ut32 bdot(ArmOp *op, ut64 addr, int k) {
 	return data;
 }
 
+/**
+ * Determine the 4-bit condition code for given string representation (e.g. "eq", "ne", ...)
+ * \p str string of at least 2 chars
+ * \return the 4-bit condition or UT8_MAX if invalid
+ */
+static ut8 parse_cond(const char *str) {
+	// clang-format off
+	switch (((ut16)str[0] << 8 | (ut16)str[1])) {
+#define COND_CASE(a, b) case (ut16)(a) << 8 | (ut16)(b)
+	COND_CASE('e', 'q'): return 0x0;
+	COND_CASE('n', 'e'): return 0x1;
+	COND_CASE('c', 's'): return 0x2;
+	COND_CASE('h', 's'): return 0x2;
+	COND_CASE('c', 'c'): return 0x3;
+	COND_CASE('l', 'o'): return 0x3;
+	COND_CASE('m', 'i'): return 0x4;
+	COND_CASE('p', 'l'): return 0x5;
+	COND_CASE('v', 's'): return 0x6;
+	COND_CASE('v', 'c'): return 0x7;
+	COND_CASE('h', 'i'): return 0x8;
+	COND_CASE('l', 's'): return 0x9;
+	COND_CASE('g', 'e'): return 0xa;
+	COND_CASE('l', 't'): return 0xb;
+	COND_CASE('g', 't'): return 0xc;
+	COND_CASE('l', 'e'): return 0xd;
+	COND_CASE('a', 'l'): return 0xe;
+	COND_CASE('n', 'v'): return 0xf;
+	default: return UT8_MAX;
+#undef COND_CASE
+	}
+	// clang-format on
+}
+
+static ut32 parse_bdot(const char *str, ArmOp *op, ut64 addr) {
+	if (!str[0] || !str[1] || str[2] != ' ') {
+	inval:
+		RZ_LOG_ERROR("assembler: arm64: invalid condition\n");
+		return UT32_MAX;
+	}
+	ut8 cond = parse_cond(str);
+	if (cond == UT8_MAX) {
+		goto inval;
+	}
+	return bdot(op, addr, 0x00000054 | ((ut32)cond << 24));
+}
+
 static ut32 mem_barrier(ArmOp *op, ut64 addr, int k) {
 	ut32 data = UT32_MAX;
 	data = k;
@@ -704,15 +750,15 @@ static ut32 msr(ArmOp *op, int w) {
 	data = 0x00000000;
 
 	if (is_immediate) {
-		//only msr has immediate mode
+		// only msr has immediate mode
 		data = 0xd500401f;
-		if (b == 0xc210) { //op0 is SPSel
-			b = 0x05; //set to immediate mode encoding
+		if (b == 0xc210) { // op0 is SPSel
+			b = 0x05; // set to immediate mode encoding
 		}
 
-		data |= (b & 0xf0) << 12; //op1
-		data |= (b & 0x0f) << 5; //op2
-		data |= (r & 0xf) << 8; //CRm(#imm)
+		data |= (b & 0xf0) << 12; // op1
+		data |= (b & 0x0f) << 5; // op2
+		data |= (r & 0xf) << 8; // CRm(#imm)
 
 	} else {
 		if (w) {
@@ -810,7 +856,7 @@ static ut32 adrp(ArmOp *op, ut64 addr, ut32 k) { //, int reg, ut64 dst) {
 	if (op->operands[0].type == ARM_GPR) {
 		data |= encode1reg(op);
 	} else {
-		eprintf("Usage: adrp x0, addr\n");
+		RZ_LOG_ERROR("assembler: arm64: adrp: invalid assembly. valid usage: adrp x0, addr\n");
 		return UT32_MAX;
 	}
 	if (op->operands[1].type == ARM_CONSTANT) {
@@ -818,7 +864,7 @@ static ut32 adrp(ArmOp *op, ut64 addr, ut32 k) { //, int reg, ut64 dst) {
 		at = op->operands[1].immediate - addr;
 		at /= 4;
 	} else {
-		eprintf("Usage: adrp, x0, addr\n");
+		RZ_LOG_ERROR("assembler: arm64: adrp: invalid assembly. valid usage: adrp x0, addr\n");
 		return UT32_MAX;
 	}
 	ut8 b0 = at;
@@ -996,19 +1042,19 @@ static bool parseOperands(char *str, ArmOp *op) {
 			token++;
 		}
 		if (operand >= MAX_OPERANDS) {
-			eprintf("Too many operands\n");
+			RZ_LOG_ERROR("assembler: arm64: maximum number of operands reached.\n");
 			return false;
 		}
 		op->operands[operand].type = ARM_NOTYPE;
 		op->operands[operand].reg_type = ARM_UNDEFINED;
 
-		//parse MSR (immediate) operand 1
+		// parse MSR (immediate) operand 1
 		if (strcmp(op->mnemonic, "msr") == 0 && operand == 1) {
 
-			//operand 1 must be a immediate
+			// operand 1 must be a immediate
 			if (token[0] == '#' || (token[0] >= '0' && token[0] <= '9')) {
-				//immediate operand found.
-				op->operands[operand].sp_val = 0xfffe; //not regiter, but a immediate
+				// immediate operand found.
+				op->operands[operand].sp_val = 0xfffe; // not regiter, but a immediate
 				op->operands[operand].immediate = rz_num_math(NULL, token[0] == '#' ? token + 1 : token);
 				operand++;
 				token = next;
@@ -1016,7 +1062,7 @@ static bool parseOperands(char *str, ArmOp *op) {
 			}
 		}
 
-		//parse system registers
+		// parse system registers
 		if ((strcmp(op->mnemonic, "mrs") == 0 && operand == 1) || (strcmp(op->mnemonic, "msr") == 0 && operand == 0)) {
 			for (msr_op_index = 0; msr_const[msr_op_index].name; msr_op_index++) {
 				if (strcasecmp(token, msr_const[msr_op_index].name) == 0) {
@@ -1223,7 +1269,7 @@ static bool parseOperands(char *str, ArmOp *op) {
 			break;
 		case '-':
 			op->operands[operand].sign = -1;
-			// falthru
+			// fallthrough
 		default:
 			op->operands_count++;
 			op->operands[operand].type = ARM_CONSTANT;
@@ -1300,6 +1346,7 @@ static bool handlePAC(ut32 *op, const char *str) {
 }
 
 bool arm64ass(const char *str, ut64 addr, ut32 *op) {
+	*op = UT32_MAX;
 	ArmOp ops = { 0 };
 	if (!parseOpcode(str, &ops)) {
 		free(ops.mnemonic);
@@ -1391,10 +1438,8 @@ bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 		*op = exception(&ops, 0x000040d4);
 	} else if (!strncmp(str, "b ", 2)) {
 		*op = branch(&ops, addr, 0x14);
-	} else if (!strncmp(str, "b.eq ", 5)) {
-		*op = bdot(&ops, addr, 0x00000054);
-	} else if (!strncmp(str, "b.hs ", 5)) {
-		*op = bdot(&ops, addr, 0x02000054);
+	} else if (!strncmp(str, "b.", 2)) {
+		*op = parse_bdot(str + 2, &ops, addr);
 	} else if (!strncmp(str, "bl ", 3)) {
 		*op = branch(&ops, addr, 0x94);
 	} else if (!strncmp(str, "br x", 4)) {
