@@ -23,7 +23,7 @@ static bool is_valid_omf_type(ut8 type) {
 			return true;
 		}
 	}
-	// eprintf ("Invalid record type\n");
+	// RZ_LOG_ERROR("Invalid record type\n");
 	return false;
 }
 
@@ -32,28 +32,28 @@ bool rz_bin_checksum_omf_ok(const ut8 *buf, ut64 buf_size) {
 	ut8 checksum = 0;
 
 	if (buf_size < 3) {
-		eprintf("Invalid record (too short)\n");
+		RZ_LOG_ERROR("Invalid record (too short)\n");
 		return false;
 	}
 	size = rz_read_le16(buf + 1);
 	if (buf_size < size + 3) {
-		eprintf("Invalid record (too short)\n");
+		RZ_LOG_ERROR("Invalid record (too short)\n");
 		return false;
 	}
-	//Some compiler set checksum to 0
+	// Some compiler set checksum to 0
 	if (!buf[size + 2]) {
 		return true;
 	}
 	size += 3;
 	for (; size; size--) {
 		if (buf_size < size) {
-			eprintf("Invalid record (too short)\n");
+			RZ_LOG_ERROR("Invalid record (too short)\n");
 			return false;
 		}
 		checksum += buf[size - 1];
 	}
 	if (checksum) {
-		// eprintf ("Invalid record checksum\n");
+		// RZ_LOG_ERROR("Invalid record checksum\n");
 	}
 	return !checksum ? true : false;
 }
@@ -110,7 +110,7 @@ static bool load_omf_lnames(OMF_record *record, const ut8 *buf, ut64 buf_size) {
 	tmp_size = 0;
 	while ((int)tmp_size < (int)(record->size - 1)) {
 		if (ct_name >= ret->nb_elem) {
-			eprintf("load_omf_lnames: prevent overflow\n");
+			RZ_LOG_ERROR("Invalid number of element (overflow)\n");
 			break;
 		}
 		// sometimes there is a name with a null size so we just skip it
@@ -121,7 +121,7 @@ static bool load_omf_lnames(OMF_record *record, const ut8 *buf, ut64 buf_size) {
 			continue;
 		}
 		if (record->size + 3 < tmp_size + cb) {
-			eprintf("Invalid Lnames record (bad size)\n");
+			RZ_LOG_ERROR("Invalid Lnames record (bad size)\n");
 			free(ret);
 			return false;
 		}
@@ -133,7 +133,7 @@ static bool load_omf_lnames(OMF_record *record, const ut8 *buf, ut64 buf_size) {
 			memcpy(names[ct_name], buf + 3 + tmp_size + 1, cb);
 		}
 		ct_name++;
-		tmp_size += cb + 1; //buf[3 + tmp_size] + 1;
+		tmp_size += cb + 1; // buf[3 + tmp_size] + 1;
 	}
 	return true;
 }
@@ -148,14 +148,14 @@ static int load_omf_segdef(OMF_record *record, const ut8 *buf, ut64 buf_size) {
 	record->content = ret;
 
 	if (record->size < 2) {
-		eprintf("Invalid Segdef record (bad size)\n");
+		RZ_LOG_ERROR("Invalid Segdef record (bad size)\n");
 		return false;
 	}
 	off_add = buf[3] & 0xe ? 0 : 3;
 
 	if (record->type == OMF_SEGDEF32) {
 		if (record->size < 5 + off_add) {
-			eprintf("Invalid Segdef record (bad size)\n");
+			RZ_LOG_ERROR("Invalid Segdef record (bad size)\n");
 			return false;
 		}
 		ret->name_idx = omf_get_idx(buf + 8 + off_add, buf_size - 8 - off_add);
@@ -166,7 +166,7 @@ static int load_omf_segdef(OMF_record *record, const ut8 *buf, ut64 buf_size) {
 		}
 	} else {
 		if (record->size < 3 + off_add) {
-			eprintf("Invalid Segdef record (bad size)\n");
+			RZ_LOG_ERROR("Invalid Segdef record (bad size)\n");
 			return false;
 		}
 		ret->name_idx = omf_get_idx(buf + 6 + off_add, buf_size - 6 - off_add);
@@ -210,20 +210,20 @@ static int load_omf_symb(OMF_record *record, ut32 ct, const ut8 *buf, int buf_si
 		symbol = ((OMF_symbol *)((OMF_multi_datas *)record->content)->elems) + nb_symb;
 
 		if (record->size - 1 < ct - 2) {
-			eprintf("Invalid Pubdef record (bad size)\n");
+			RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 			return false;
 		}
 		str_size = buf[ct];
 
 		if (bits == 32) {
 			if (ct + 1 + str_size + 4 - 3 > record->size) {
-				eprintf("Invalid Pubdef record (bad size)\n");
+				RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 				return false;
 			}
 			symbol->offset = rz_read_le32(buf + ct + 1 + str_size);
 		} else {
 			if (ct + 1 + str_size + 2 - 3 > record->size) {
-				eprintf("Invalid Pubdef record (bad size)\n");
+				RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 				return false;
 			}
 			symbol->offset = rz_read_le16(buf + ct + 1 + str_size);
@@ -241,7 +241,7 @@ static int load_omf_symb(OMF_record *record, ut32 ct, const ut8 *buf, int buf_si
 		if (ct >= buf_size) {
 			return false;
 		}
-		if (buf[ct] & 0x80) { //type index
+		if (buf[ct] & 0x80) { // type index
 			ct += 2;
 		} else {
 			ct++;
@@ -258,7 +258,7 @@ static int load_omf_pubdef(OMF_record *record, const ut8 *buf, int buf_size) {
 	ut16 base_grp;
 
 	if (record->size < 2) {
-		eprintf("Invalid Pubdef record (bad size)\n");
+		RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 		return false;
 	}
 
@@ -271,7 +271,7 @@ static int load_omf_pubdef(OMF_record *record, const ut8 *buf, int buf_size) {
 	}
 
 	if (record->size < ct - 2) {
-		eprintf("Invalid Pubdef record (bad size)\n");
+		RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 		return false;
 	}
 
@@ -287,7 +287,7 @@ static int load_omf_pubdef(OMF_record *record, const ut8 *buf, int buf_size) {
 		ct += 2;
 	}
 	if (record->size < ct - 2) {
-		eprintf("Invalid Pubdef record (bad size)\n");
+		RZ_LOG_ERROR("Invalid Pubdef record (bad size)\n");
 		return false;
 	}
 
@@ -330,13 +330,13 @@ static int load_omf_data(const ut8 *buf, int buf_size, OMF_record *record, ut64
 	OMF_data *ret;
 
 	if ((!(record->type & 1) && record->size < 4) || (record->size < 6)) {
-		eprintf("Invalid Ledata record (bad size)\n");
+		RZ_LOG_ERROR("Invalid Ledata record (bad size)\n");
 		return false;
 	}
 	seg_idx = omf_get_idx(buf + 3, buf_size - 3);
 	if (seg_idx & 0xff00) {
 		if ((!(record->type & 1) && record->size < 5) || (record->size < 7)) {
-			eprintf("Invalid Ledata record (bad size)\n");
+			RZ_LOG_ERROR("Invalid Ledata record (bad size)\n");
 			return false;
 		}
 		ct++;
@@ -378,7 +378,7 @@ static int load_omf_content(OMF_record *record, const ut8 *buf, ut64 global_ct,
 	}
 	// generic loader just copy data from buf to content
 	if (!record->size) {
-		eprintf("Invalid record (size to short)\n");
+		RZ_LOG_ERROR("Invalid record (size to short)\n");
 		return false;
 	}
 	if (!(record->content = RZ_NEWS0(char, record->size))) {
@@ -400,7 +400,7 @@ static OMF_record_handler *load_record_omf(const ut8 *buf, ut64 global_ct, ut64
 
 		// at least a record have a type a size and a checksum
 		if (((OMF_record *)new)->size > buf_size - 3 || buf_size < 4) {
-			eprintf("Invalid record (too short)\n");
+			RZ_LOG_ERROR("Invalid record (too short)\n");
 			RZ_FREE(new);
 			return NULL;
 		}
@@ -544,7 +544,7 @@ static int get_omf_data_info(rz_bin_omf_obj *obj) {
 
 	while ((tmp = get_next_omf_record_type(tmp, OMF_LEDATA))) {
 		if (((OMF_data *)((OMF_record *)tmp)->content)->seg_idx - 1 >= obj->nb_section) {
-			eprintf("Invalid Ledata record (bad segment index)\n");
+			RZ_LOG_ERROR("Invalid Ledata record (bad segment index)\n");
 			return false;
 		}
 		OMF_segment *os = obj->sections[((OMF_data *)((OMF_record *)tmp)->content)->seg_idx - 1];
@@ -709,7 +709,7 @@ bool rz_bin_omf_get_entry(rz_bin_omf_obj *obj, RzBinAddr *addr) {
 	while (ct_sym < obj->nb_symbol) {
 		if (!strcmp(obj->symbols[ct_sym]->name, "_start")) {
 			if (obj->symbols[ct_sym]->seg_idx - 1 > obj->nb_section) {
-				eprintf("Invalid segment index for symbol _start\n");
+				RZ_LOG_ERROR("Invalid segment index for symbol _start\n");
 				return false;
 			}
 			addr->vaddr = obj->sections[obj->symbols[ct_sym]->seg_idx - 1]->vaddr + obj->symbols[ct_sym]->offset + OMF_BASE_ADDR;
@@ -799,7 +799,7 @@ ut64 rz_bin_omf_get_vaddr_sym(rz_bin_omf_obj *obj, OMF_symbol *sym) {
 		return 0LL;
 	}
 	if (sym->seg_idx >= obj->nb_section) {
-		eprintf("Invalid segment index for symbol %s\n", sym->name);
+		RZ_LOG_ERROR("Invalid segment index for symbol %s\n", sym->name);
 		return 0;
 	}
 	if (sym->seg_idx == 0) {
