@@ -23,9 +23,9 @@ RZ_API int rz_core_setup_debugger(RzCore *r, const char *debugbackend, bool atta
 	if (attach) {
 		rz_core_debug_attach(r, pid);
 	}
-	//this makes to attach twice showing warnings in the output
-	//we get "resource busy" so it seems isn't an issue
-	rz_core_debug_regs2flags(r, 0);
+	// this makes to attach twice showing warnings in the output
+	// we get "resource busy" so it seems isn't an issue
+	rz_core_reg_update_flags(r);
 	/* honor dbg.bep */
 	{
 		const char *bep = rz_config_get(r->config, "dbg.bep");
@@ -92,221 +92,6 @@ RZ_API bool rz_core_dump(RzCore *core, const char *file, ut64 addr, ut64 size, i
 	return true;
 }
 
-static bool __endian_swap(ut8 *buf, ut32 blocksize, ut8 len) {
-	ut32 i;
-	ut16 v16;
-	ut32 v32;
-	ut64 v64;
-	if (len != 8 && len != 4 && len != 2 && len != 1) {
-		eprintf("Invalid word size. Use 1, 2, 4 or 8\n");
-		return false;
-	}
-	if (len == 1) {
-		return true;
-	}
-	for (i = 0; i < blocksize; i += len) {
-		switch (len) {
-		case 8:
-			v64 = rz_read_at_be64(buf, i);
-			rz_write_at_le64(buf, v64, i);
-			break;
-		case 4:
-			v32 = rz_read_at_be32(buf, i);
-			rz_write_at_le32(buf, v32, i);
-			break;
-		case 2:
-			v16 = rz_read_at_be16(buf, i);
-			rz_write_at_le16(buf, v16, i);
-			break;
-		}
-	}
-	return true;
-}
-
-RZ_API ut8 *rz_core_transform_op(RzCore *core, const char *arg, char op) {
-	int i, j;
-	ut64 len;
-	char *str = NULL;
-	ut8 *buf;
-
-	buf = (ut8 *)malloc(core->blocksize);
-	if (!buf) {
-		return NULL;
-	}
-	memcpy(buf, core->block, core->blocksize);
-
-	if (op != 'e') {
-		// fill key buffer either from arg or from clipboard
-		if (arg) { // parse arg for key
-			// rz_hex_str2bin() is guaranteed to output maximum half the
-			// input size, or 1 byte if there is just a single nibble.
-			str = (char *)malloc(strlen(arg) / 2 + 1);
-			if (!str) {
-				goto beach;
-			}
-			len = rz_hex_str2bin(arg, (ut8 *)str);
-			// Output is invalid if there was just a single nibble,
-			// but in that case, len is negative (-1).
-			if (len <= 0) {
-				eprintf("Invalid hexpair string\n");
-				goto beach;
-			}
-		} else { // use clipboard as key
-			const ut8 *tmp = rz_buf_data(core->yank_buf, &len);
-			str = rz_mem_dup(tmp, len);
-			if (!str) {
-				goto beach;
-			}
-		}
-	} else {
-		len = 0;
-	}
-
-	// execute the operand
-	if (op == 'e') {
-		int wordsize = 1;
-		char *os, *p, *s = strdup(arg);
-		int n = 0, from = 0, to = UT8_MAX, dif = 0, step = 1;
-		os = s;
-		p = strchr(s, ' ');
-		if (p) {
-			*p = 0;
-			from = rz_num_math(core->num, s);
-			s = p + 1;
-		}
-		p = strchr(s, ' ');
-		if (p) {
-			*p = 0;
-			to = rz_num_math(core->num, s);
-			s = p + 1;
-		}
-		p = strchr(s, ' ');
-		if (p) {
-			*p = 0;
-			step = rz_num_math(core->num, s);
-			s = p + 1;
-			wordsize = rz_num_math(core->num, s);
-		} else {
-			step = rz_num_math(core->num, s);
-		}
-		free(os);
-		eprintf("from %d to %d step %d size %d\n", from, to, step, wordsize);
-		dif = (to <= from) ? UT8_MAX : to - from + 1;
-		if (wordsize == 1) {
-			from %= (UT8_MAX + 1);
-		}
-		if (dif < 1) {
-			dif = UT8_MAX + 1;
-		}
-		if (step < 1) {
-			step = 1;
-		}
-		if (wordsize < 1) {
-			wordsize = 1;
-		}
-		if (wordsize == 1) {
-			for (i = n = 0; i < core->blocksize; i++, n += step) {
-				buf[i] = (ut8)(n % dif) + from;
-			}
-		} else if (wordsize == 2) {
-			ut16 num16 = from;
-			for (i = 0; i < core->blocksize; i += wordsize, num16 += step) {
-				rz_write_le16(buf + i, num16);
-			}
-		} else if (wordsize == 4) {
-			ut32 num32 = from;
-			for (i = 0; i < core->blocksize; i += wordsize, num32 += step) {
-				rz_write_le32(buf + i, num32);
-			}
-		} else if (wordsize == 8) {
-			ut64 num64 = from;
-			for (i = 0; i < core->blocksize; i += wordsize, num64 += step) {
-				rz_write_le64(buf + i, num64);
-			}
-		} else {
-			eprintf("Invalid word size. Use 1, 2, 4 or 8\n");
-		}
-	} else if (op == '2' || op == '4' || op == '8') { // "wo2" "wo4" "wo8"
-		int inc = op - '0';
-		ut8 tmp;
-		for (i = 0; (i + inc) <= core->blocksize; i += inc) {
-			if (inc == 2) {
-				tmp = buf[i];
-				buf[i] = buf[i + 1];
-				buf[i + 1] = tmp;
-			} else if (inc == 4) {
-				tmp = buf[i];
-				buf[i] = buf[i + 3];
-				buf[i + 3] = tmp;
-				tmp = buf[i + 1];
-				buf[i + 1] = buf[i + 2];
-				buf[i + 2] = tmp;
-			} else if (inc == 8) {
-				tmp = buf[i];
-				buf[i] = buf[i + 7];
-				buf[i + 7] = tmp;
-
-				tmp = buf[i + 1];
-				buf[i + 1] = buf[i + 6];
-				buf[i + 6] = tmp;
-
-				tmp = buf[i + 2];
-				buf[i + 2] = buf[i + 5];
-				buf[i + 5] = tmp;
-
-				tmp = buf[i + 3];
-				buf[i + 3] = buf[i + 4];
-				buf[i + 4] = tmp;
-			} else {
-				eprintf("Invalid inc, use 2, 4 or 8.\n");
-				break;
-			}
-		}
-	} else {
-		bool be = rz_config_get_i(core->config, "cfg.bigendian");
-		if (!be) {
-			if (!__endian_swap((ut8 *)str, len, len)) {
-				goto beach;
-			}
-		}
-		for (i = j = 0; i < core->blocksize; i++) {
-			switch (op) {
-			case 'x': buf[i] ^= str[j]; break;
-			case 'a': buf[i] += str[j]; break;
-			case 's': buf[i] -= str[j]; break;
-			case 'm': buf[i] *= str[j]; break;
-			case 'w': buf[i] = str[j]; break;
-			case 'd': buf[i] = (str[j]) ? (buf[i] / str[j]) : 0; break;
-			case 'r': buf[i] >>= str[j]; break;
-			case 'l': buf[i] <<= str[j]; break;
-			case 'o': buf[i] |= str[j]; break;
-			case 'A': buf[i] &= str[j]; break;
-			}
-			j++;
-			if (j >= len) {
-				j = 0; /* cyclic key */
-			}
-		}
-	}
-
-	free(str);
-	return buf;
-beach:
-	free(str);
-	free(buf);
-	return NULL;
-}
-
-RZ_API int rz_core_write_op(RzCore *core, const char *arg, char op) {
-	ut8 *buf = rz_core_transform_op(core, arg, op);
-	if (!buf) {
-		return false;
-	}
-	int ret = rz_core_write_at(core, core->offset, buf, core->blocksize);
-	free(buf);
-	return ret;
-}
-
 // Get address-specific bits and arch at a certain address.
 // If there are no specific infos (i.e. asm.bits and asm.arch should apply), the bits and arch will be 0 or NULL respectively!
 RZ_API void rz_core_arch_bits_at(RzCore *core, ut64 addr, RZ_OUT RZ_NULLABLE int *bits, RZ_OUT RZ_BORROW RZ_NULLABLE const char **arch) {
@@ -329,7 +114,7 @@ RZ_API void rz_core_arch_bits_at(RzCore *core, ut64 addr, RZ_OUT RZ_NULLABLE int
 			}
 		}
 	}
-	//if we found bits related with analysis hints pick it up
+	// if we found bits related with analysis hints pick it up
 	if (bits && !bitsval && !core->fixedbits) {
 		bitsval = rz_analysis_hint_bits_at(core->analysis, addr, NULL);
 	}
@@ -366,30 +151,50 @@ RZ_API bool rz_core_write_at(RzCore *core, ut64 addr, const ut8 *buf, int size)
 	if (addr >= core->offset && addr <= core->offset + core->blocksize - 1) {
 		rz_core_block_read(core);
 	}
+	if (rz_config_get_i(core->config, "cfg.wseek")) {
+		rz_core_seek_delta(core, size, true);
+	}
 	return ret;
 }
 
-RZ_API bool rz_core_extend_at(RzCore *core, ut64 addr, int size) {
-	if (!core->io || !core->file || size < 1) {
-		return false;
-	}
+/**
+ * \brief Extend the file at current offset by inserting \p size 0 bytes at \p addr
+ *
+ * \p addr is an physical/virtual address based on the value of eval "io.va".
+ * When virtual it is translated to a physical address according to the IO map
+ * at the current offset
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Address where to insert new 0 bytes.
+ * \param size Number of 0 bytes to insert
+ * \return true if extend operation was successful, false otherwise
+ */
+RZ_API bool rz_core_extend_at(RzCore *core, ut64 addr, ut64 size) {
+	rz_return_val_if_fail(core, false);
+
 	int io_va = rz_config_get_i(core->config, "io.va");
 	if (io_va) {
 		RzIOMap *map = rz_io_map_get(core->io, core->offset);
 		if (map) {
 			addr = addr - map->itv.addr + map->delta;
 		}
-		rz_config_set_i(core->config, "io.va", false);
 	}
-	int ret = rz_io_extend_at(core->io, addr, size);
-	if (addr >= core->offset && addr <= core->offset + core->blocksize) {
-		rz_core_block_read(core);
-	}
-	rz_config_set_i(core->config, "io.va", io_va);
+	bool ret = rz_io_extend_at(core->io, addr, size);
+	rz_core_block_read(core);
 	return ret;
 }
 
-RZ_API int rz_core_shift_block(RzCore *core, ut64 addr, ut64 b_size, st64 dist) {
+/**
+ * \brief Shift a block of data from \p addr of size \p b_size left or right based on \p dist.
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Address of the block of data to move
+ * \param b_size Size of the block of data to move
+ * \param dist Where to shift the data, whether backward or forward and how
+ *             distant from the original position
+ * \return true if the shift operation was succesful, false otherwise
+ */
+RZ_API bool rz_core_shift_block(RzCore *core, ut64 addr, ut64 b_size, st64 dist) {
 	// bstart - block start, fstart file start
 	ut64 fend = 0, fstart = 0, bstart = 0, file_sz = 0;
 	ut8 *shift_buf = NULL;
@@ -420,19 +225,8 @@ RZ_API int rz_core_shift_block(RzCore *core, ut64 addr, ut64 b_size, st64 dist)
 		return false;
 	}
 
-	// cases
-	// addr + b_size + dist > file_end
-	//if ( (addr+b_size) + dist > file_end ) {
-	//	res = false;
-	//}
-	// addr + b_size + dist < file_start (should work since dist is signed)
-	//else if ( (addr+b_size) + dist < 0 ) {
-	//	res = false;
-	//}
-	// addr + dist < file_start
 	if (addr + dist < fstart) {
 		res = false;
-		// addr + dist > file_end
 	} else if ((addr) + dist > fend) {
 		res = false;
 	} else {
@@ -478,103 +272,148 @@ RZ_API int rz_core_write_hexpair(RzCore *core, ut64 addr, const char *pairs) {
 		return 0;
 	}
 	int len = rz_hex_str2bin(pairs, buf);
-	if (len != 0) {
-		if (len < 0) {
-			len = -len;
-			if (len < core->blocksize) {
-				buf[len - 1] |= core->block[len - 1] & 0xf;
-			}
-		}
-		core->num->value = 0;
-		if (!rz_core_write_at(core, addr, buf, len)) {
-			eprintf("Failed to write\n");
-			core->num->value = 1;
-		}
-		if (rz_config_get_i(core->config, "cfg.wseek")) {
-			rz_core_seek_delta(core, len, true);
-		}
-		rz_core_block_read(core);
-	} else {
-		eprintf("Error: invalid hexpair string\n");
-		core->num->value = 1;
+	if (len < 0) {
+		RZ_LOG_ERROR("Could not convert hexpair '%s' to bin data\n", pairs);
+		goto err;
 	}
+	if (!rz_core_write_at(core, addr, buf, len)) {
+		RZ_LOG_ERROR("Could not write hexpair '%s' at %" PFMT64x "\n", pairs, addr);
+		goto err;
+	}
+err:
 	free(buf);
 	return len;
 }
 
 /**
- * Assembles instructions and writes the resulting data at the given offset.
+ * Writes the bytes \p data at address \p addr cyclically until it fills the whole block
  *
- * Returns the length of the written data or -1 in case of error
+ * It repeats the data \p data with length \p len until it fills an entire block
+ * starting at \p addr.
+ *
+ * \param core RzCore reference
+ * \param addr Address to where to write
+ * \param data Array of bytes to cyclically write in the block at \p addr
+ * \param len Length of \p data
+ */
+RZ_API bool rz_core_write_block(RzCore *core, ut64 addr, ut8 *data, size_t len) {
+	rz_return_val_if_fail(core && data, 0);
+
+	ut8 *buf = RZ_NEWS(ut8, core->blocksize);
+	if (!buf) {
+		return false;
+	}
+
+	bool res = false;
+	rz_mem_copyloop(buf, data, core->blocksize, len);
+	if (!rz_core_write_at(core, addr, buf, core->blocksize)) {
+		RZ_LOG_ERROR("Could not write cyclic data (%d bytes) at %" PFMT64x "\n", core->blocksize, addr);
+		goto err;
+	}
+	res = true;
+err:
+	free(buf);
+	return res;
+}
+
+/**
+ * \brief Assembles instructions and writes the resulting data at the given offset.
  *
  * \param core RzCore reference
  * \param addr Address to where to write
  * \param instructions List of instructions to assemble as a string
- * \param pretend Don't write but emit the sequence of `wx` commands
- * \param pad Fit the instruction inside the current instruction, fill with nops to pad
+ * \return Returns the length of the written data or -1 in case of error
  */
-RZ_API int rz_core_write_assembly(RzCore *core, ut64 addr, const char *instructions, bool pretend, bool pad) {
-	int wseek = rz_config_get_i(core->config, "cfg.wseek");
+RZ_API int rz_core_write_assembly(RzCore *core, ut64 addr, const char *instructions) {
+	rz_return_val_if_fail(core && instructions, -1);
+
+	int ret = -1;
+
 	rz_asm_set_pc(core->rasm, core->offset);
 	RzAsmCode *acode = rz_asm_massemble(core->rasm, instructions);
 	if (!acode) {
 		return -1;
 	}
-	if (pad) { // "wai"
-		RzAnalysisOp analop;
-		if (!rz_analysis_op(core->analysis, &analop, core->offset, core->block, core->blocksize, RZ_ANALYSIS_OP_MASK_BASIC)) {
-			eprintf("Invalid instruction?\n");
-			return -1;
-		}
-		if (analop.size < acode->len) {
-			eprintf("Doesnt fit\n");
-			rz_analysis_op_fini(&analop);
-			rz_asm_code_free(acode);
-			return -1;
-		}
-		rz_analysis_op_fini(&analop);
-		rz_core_hack(core, "nop");
+	if (acode->len <= 0) {
+		ret = 0;
+		goto err;
 	}
-	if (acode->len > 0) {
-		char *hex = rz_asm_code_get_hex(acode);
-		if (pretend) {
-			rz_cons_printf("wx %s\n", hex);
-		} else {
-			if (!rz_core_write_at(core, core->offset, acode->bytes, acode->len)) {
-				eprintf("Failed to write %d bytes at 0x%" PFMT64x "address\n", acode->len, core->offset);
-				core->num->value = 1;
-				free(hex);
-				return -1;
-			} else {
-				if (rz_config_get_i(core->config, "scr.prompt")) {
-					eprintf("Written %d byte(s) (%s) = wx %s\n", acode->len, instructions, hex);
-				}
-				if (wseek) {
-					rz_core_seek_delta(core, acode->len, true);
-				}
-			}
-			rz_core_block_read(core);
-		}
-		free(hex);
-		return acode->len;
-	} else {
-		eprintf("Nothing to do.\n");
-		return 0;
+
+	if (!rz_core_write_at(core, core->offset, acode->bytes, acode->len)) {
+		RZ_LOG_ERROR("Cannot write %d bytes at 0x%" PFMT64x "address\n", acode->len, core->offset);
+		core->num->value = 1;
+		goto err;
 	}
+	ret = acode->len;
+err:
 	rz_asm_code_free(acode);
-	return -1;
+	return ret;
 }
 
+/**
+ * \brief Assemble instructions and write the resulting data inside the current instruction.
+ *
+ * Assemble one or more instructions and write the resulting data inside the
+ * current instruction, if the new instructions fit. Fill the rest of the bytes
+ * of the old instruction with NOP
+ *
+ * \param core RzCore reference
+ * \param addr Address to where to write
+ * \param instructions List of instructions to assemble as a string
+ * \return Returns the length of the written data or -1 in case of error (e.g. the new instruction does not fit)
+ */
+RZ_API int rz_core_write_assembly_fill(RzCore *core, ut64 addr, const char *instructions) {
+	rz_return_val_if_fail(core && instructions, -1);
+
+	int ret = -1;
+
+	rz_asm_set_pc(core->rasm, core->offset);
+	RzAsmCode *acode = rz_asm_massemble(core->rasm, instructions);
+	if (!acode) {
+		return -1;
+	}
+	if (acode->len <= 0) {
+		ret = 0;
+		goto err;
+	}
+
+	RzAnalysisOp op = { 0 };
+	if (!rz_analysis_op(core->analysis, &op, core->offset, core->block, core->blocksize, RZ_ANALYSIS_OP_MASK_BASIC)) {
+		RZ_LOG_ERROR("Invalid instruction at %" PFMT64x "\n", core->offset);
+		goto err;
+	}
+	if (op.size < acode->len) {
+		RZ_LOG_ERROR("Instructions do not fit at %" PFMT64x "\n", core->offset);
+		goto err;
+	}
+	rz_core_hack(core, "nop");
+
+	if (!rz_core_write_at(core, core->offset, acode->bytes, acode->len)) {
+		RZ_LOG_ERROR("Cannot write %d bytes at 0x%" PFMT64x "address\n", acode->len, core->offset);
+		core->num->value = 1;
+		goto err;
+	}
+	ret = acode->len;
+err:
+	rz_asm_code_free(acode);
+	return ret;
+}
+
+/**
+ * \brief Print an IO plugin according to \p state
+ *
+ * \param plugin Reference to RzIOPlugin
+ * \param state Specify how the plugin shall be printed
+ */
 RZ_API RzCmdStatus rz_core_io_plugin_print(RzIOPlugin *plugin, RzCmdStateOutput *state) {
 	char str[4];
 	PJ *pj = state->d.pj;
+	str[0] = 'r';
+	str[1] = plugin->write ? 'w' : '_';
+	str[2] = plugin->isdbg ? 'd' : '_';
+	str[3] = 0;
 	switch (state->mode) {
-	case RZ_OUTPUT_MODE_JSON: {
-		str[0] = 'r';
-		str[1] = plugin->write ? 'w' : '_';
-		str[2] = plugin->isdbg ? 'd' : '_';
-		str[3] = 0;
-
+	case RZ_OUTPUT_MODE_JSON:
 		pj_o(pj);
 		pj_ks(pj, "permissions", str);
 		pj_ks(pj, "name", plugin->name);
@@ -603,12 +442,14 @@ RZ_API RzCmdStatus rz_core_io_plugin_print(RzIOPlugin *plugin, RzCmdStateOutput
 		}
 		pj_end(pj);
 		break;
-	}
-	case RZ_OUTPUT_MODE_STANDARD: {
-		str[0] = 'r';
-		str[1] = plugin->write ? 'w' : '_';
-		str[2] = plugin->isdbg ? 'd' : '_';
-		str[3] = 0;
+	case RZ_OUTPUT_MODE_TABLE:
+		rz_table_set_columnsf(state->d.t, "sssss", "perm", "license", "name", "uri", "description");
+		rz_table_add_rowf(state->d.t, "sssss", str, plugin->license, plugin->name, plugin->uris, plugin->desc);
+		break;
+	case RZ_OUTPUT_MODE_QUIET:
+		rz_cons_printf("%s\n", plugin->name);
+		break;
+	case RZ_OUTPUT_MODE_STANDARD:
 		rz_cons_printf("%s  %-8s %s (%s)",
 			str, plugin->name,
 			plugin->desc, plugin->license);
@@ -623,7 +464,6 @@ RZ_API RzCmdStatus rz_core_io_plugin_print(RzIOPlugin *plugin, RzCmdStateOutput
 		}
 		rz_cons_printf("\n");
 		break;
-	}
 	default: {
 		rz_warn_if_reached();
 		return RZ_CMD_STATUS_NONEXISTINGCMD;
@@ -632,6 +472,12 @@ RZ_API RzCmdStatus rz_core_io_plugin_print(RzIOPlugin *plugin, RzCmdStateOutput
 	return RZ_CMD_STATUS_OK;
 }
 
+/**
+ * \brief Print the registered IO plugins according to \p state
+ *
+ * \param io Reference to RzIO instance
+ * \param state Specify how plugins shall be printed
+ */
 RZ_API RzCmdStatus rz_core_io_plugins_print(RzIO *io, RzCmdStateOutput *state) {
 	RzIOPlugin *plugin;
 	RzListIter *iter;
@@ -639,6 +485,7 @@ RZ_API RzCmdStatus rz_core_io_plugins_print(RzIO *io, RzCmdStateOutput *state) {
 		return RZ_CMD_STATUS_ERROR;
 	}
 	rz_cmd_state_output_array_start(state);
+	rz_cmd_state_output_set_columnsf(state, "sssss", "perm", "license", "name", "uri", "description");
 	rz_list_foreach (io->plugins, iter, plugin) {
 		rz_core_io_plugin_print(plugin, state);
 	}
@@ -688,10 +535,6 @@ RZ_API bool rz_core_write_value_at(RzCore *core, ut64 addr, ut64 value, int sz)
 		core->num->value = 1;
 		return false;
 	}
-	if (rz_config_get_i(core->config, "cfg.wseek")) {
-		rz_core_seek_delta(core, sz, true);
-	}
-
 	return true;
 }
 
@@ -749,10 +592,6 @@ RZ_API bool rz_core_write_value_inc_at(RzCore *core, ut64 addr, st64 value, int
 		RZ_LOG_ERROR("Could not write %d bytes at %" PFMT64x "\n", sz, addr);
 		return false;
 	}
-	if (rz_config_get_i(core->config, "cfg.wseek")) {
-		rz_core_seek_delta(core, sz, true);
-	}
-
 	return true;
 }
 
@@ -774,11 +613,80 @@ RZ_API bool rz_core_write_string_at(RzCore *core, ut64 addr, const char *s) {
 	int len = rz_str_unescape(str);
 	if (!rz_core_write_at(core, addr, (const ut8 *)str, len)) {
 		RZ_LOG_ERROR("Could not write '%s' at %" PFMT64x "\n", s, addr);
+		free(str);
 		return false;
 	}
-	if (rz_config_get_i(core->config, "cfg.wseek")) {
-		rz_core_seek_delta(core, len, true);
+	free(str);
+	return true;
+}
+
+/**
+ * \brief Write a given string \p s as a wide string at the specified \p addr
+ *
+ * \param core RzCore reference
+ * \param addr Address where to write the string
+ * \param s String to write. The string is unescaped, meaning that if there is `\n` it becomes 0x0a
+ */
+RZ_API bool rz_core_write_string_wide_at(RzCore *core, ut64 addr, const char *s) {
+	rz_return_val_if_fail(core && s, false);
+
+	bool res = false;
+	char *str = strdup(s);
+	if (!str) {
+		return false;
 	}
+
+	int len = rz_str_unescape(str);
+	if (len < 1) {
+		goto str_err;
+	}
+
+	len++; // Consider for the terminator char
+	char *tmp = RZ_NEWS(char, len * 2);
+	if (!tmp) {
+		goto str_err;
+	}
+
+	for (int i = 0; i < len; i++) {
+		tmp[i * 2] = str[i];
+		tmp[i * 2 + 1] = 0;
+	}
+
+	if (!rz_core_write_at(core, addr, (const ut8 *)tmp, len * 2)) {
+		RZ_LOG_ERROR("Could not write wide string '%s' at %" PFMT64x "\n", s, addr);
+		free(str);
+		return false;
+	}
+	res = true;
+str_err:
+	free(str);
+	return res;
+}
+/**
+ * \brief Write at the specified \p addr the length of the string in one byte,
+ * followed by the given string \p s
+ *
+ * \param core RzCore reference
+ * \param addr Address where to write the string
+ * \param s String to write. The string is unescaped, meaning that if there is `\n` it becomes 0x0a
+ */
+RZ_API bool rz_core_write_length_string_at(RzCore *core, ut64 addr, const char *s) {
+	rz_return_val_if_fail(core && s, false);
+
+	char *str = strdup(s);
+	if (!str) {
+		return false;
+	}
+
+	int len = rz_str_unescape(str);
+	ut8 ulen = (ut8)len;
+	if (!rz_core_write_at(core, addr, &ulen, sizeof(ulen)) ||
+		!rz_core_write_at(core, addr + 1, (const ut8 *)str, len)) {
+		RZ_LOG_ERROR("Could not write length+'%s' at %" PFMT64x "\n", s, addr);
+		free(str);
+		return false;
+	}
+	free(str);
 	return true;
 }
 
@@ -821,11 +729,7 @@ RZ_API bool rz_core_write_base64_at(RzCore *core, ut64 addr, const char *s) {
 		RZ_LOG_ERROR("Could not write base64 encoded string '%s' at %" PFMT64x "\n", s, addr);
 		goto err;
 	}
-	if (rz_config_get_i(core->config, "cfg.wseek")) {
-		rz_core_seek_delta(core, len, true);
-	}
 	res = true;
-
 err:
 	free(buf);
 	return res;
@@ -853,12 +757,373 @@ RZ_API bool rz_core_write_base64d_at(RzCore *core, ut64 addr, const char *s) {
 		RZ_LOG_ERROR("Could not write base64 decoded string '%s' at %" PFMT64x "\n", s, addr);
 		goto err;
 	}
-	if (rz_config_get_i(core->config, "cfg.wseek")) {
-		rz_core_seek_delta(core, len, true);
-	}
 	res = true;
-
 err:
 	free(buf);
 	return res;
 }
+
+/**
+ * \brief Write \p len random bytes at address \p addr
+ *
+ * \param core RzCore reference
+ * \param addr Address where to write the data
+ * \param len Length of the random data to write
+ */
+RZ_API bool rz_core_write_random_at(RzCore *core, ut64 addr, size_t len) {
+	rz_return_val_if_fail(core, false);
+
+	bool res = false;
+	ut8 *buf = malloc(len);
+	if (!buf) {
+		return false;
+	}
+
+	rz_num_irand();
+	for (int i = 0; i < len; i++) {
+		buf[i] = rz_num_rand(256);
+	}
+
+	if (!rz_core_write_at(core, addr, buf, len)) {
+		RZ_LOG_ERROR("Could not write random data of length %zd at %" PFMT64x "\n", len, addr);
+		goto err;
+	}
+	res = true;
+err:
+	free(buf);
+	return res;
+}
+
+RZ_API RzCmdStatus rz_core_io_cache_print(RzCore *core, RzCmdStateOutput *state) {
+	rz_return_val_if_fail(core && core->io, RZ_CMD_STATUS_ERROR);
+
+	size_t i, j = 0;
+	void **iter;
+	RzIOCache *c;
+
+	rz_pvector_foreach (&core->io->cache, iter) {
+		c = *iter;
+		const ut64 dataSize = rz_itv_size(c->itv);
+		switch (state->mode) {
+		case RZ_OUTPUT_MODE_STANDARD:
+			rz_cons_printf("idx=%" PFMTSZu " addr=0x%08" PFMT64x " size=%" PFMT64u " ", j, rz_itv_begin(c->itv), dataSize);
+			for (i = 0; i < dataSize; i++) {
+				rz_cons_printf("%02x", c->odata[i]);
+			}
+			rz_cons_printf(" -> ");
+			for (i = 0; i < dataSize; i++) {
+				rz_cons_printf("%02x", c->data[i]);
+			}
+			rz_cons_printf(" %s\n", c->written ? "(written)" : "(not written)");
+			break;
+		case RZ_OUTPUT_MODE_JSON:
+			pj_o(state->d.pj);
+			pj_kn(state->d.pj, "idx", j);
+			pj_kn(state->d.pj, "addr", rz_itv_begin(c->itv));
+			pj_kn(state->d.pj, "size", dataSize);
+			char *hex = rz_hex_bin2strdup(c->odata, dataSize);
+			pj_ks(state->d.pj, "before", hex);
+			free(hex);
+			hex = rz_hex_bin2strdup(c->data, dataSize);
+			pj_ks(state->d.pj, "after", hex);
+			free(hex);
+			pj_kb(state->d.pj, "written", c->written);
+			pj_end(state->d.pj);
+			break;
+		case RZ_OUTPUT_MODE_RIZIN:
+			rz_cons_printf("wx ");
+			for (i = 0; i < dataSize; i++) {
+				rz_cons_printf("%02x", (ut8)(c->data[i] & 0xff));
+			}
+			rz_cons_printf(" @ 0x%08" PFMT64x, rz_itv_begin(c->itv));
+			rz_cons_printf(" # replaces: ");
+			for (i = 0; i < dataSize; i++) {
+				rz_cons_printf("%02x", (ut8)(c->odata[i] & 0xff));
+			}
+			rz_cons_printf("\n");
+			break;
+		default:
+			rz_warn_if_reached();
+			break;
+		}
+		j++;
+	}
+	return RZ_CMD_STATUS_OK;
+}
+
+RZ_API RzCmdStatus rz_core_io_pcache_print(RzCore *core, RzIODesc *desc, RzCmdStateOutput *state) {
+	rz_return_val_if_fail(core && core->io, RZ_CMD_STATUS_ERROR);
+	rz_return_val_if_fail(desc, RZ_CMD_STATUS_ERROR);
+
+	RzList *caches = rz_io_desc_cache_list(desc);
+	RzListIter *iter;
+	RzIOCache *c;
+
+	if (state->mode == RZ_OUTPUT_MODE_RIZIN) {
+		rz_cons_printf("e io.va = false\n");
+	}
+	rz_list_foreach (caches, iter, c) {
+		const int cacheSize = rz_itv_size(c->itv);
+		int i;
+
+		switch (state->mode) {
+		case RZ_OUTPUT_MODE_STANDARD:
+			rz_cons_printf("0x%08" PFMT64x ": %02x",
+				rz_itv_begin(c->itv), c->odata[0]);
+			for (i = 1; i < cacheSize; i++) {
+				rz_cons_printf("%02x", c->odata[i]);
+			}
+			rz_cons_printf(" -> %02x", c->data[0]);
+			for (i = 1; i < cacheSize; i++) {
+				rz_cons_printf("%02x", c->data[i]);
+			}
+			rz_cons_printf("\n");
+			break;
+		case RZ_OUTPUT_MODE_RIZIN:
+			rz_cons_printf("wx %02x", c->data[0]);
+			for (i = 1; i < cacheSize; i++) {
+				rz_cons_printf("%02x", c->data[i]);
+			}
+			rz_cons_printf(" @ 0x%08" PFMT64x " \n", rz_itv_begin(c->itv));
+			break;
+		default:
+			rz_warn_if_reached();
+			break;
+		}
+	}
+	rz_list_free(caches);
+	return RZ_CMD_STATUS_OK;
+}
+
+/**
+ * \brief Write a given string \p s, followed by the zero terminator, at the specified \p addr
+ *
+ * \param core RzCore reference
+ * \param addr Address where to write the string
+ * \param s String to write. The string is unescaped, meaning that if there is `\n` it becomes 0x0a
+ */
+RZ_API bool rz_core_write_string_zero_at(RzCore *core, ut64 addr, const char *s) {
+	rz_return_val_if_fail(core && s, false);
+
+	char *str = strdup(s);
+	if (!str) {
+		return false;
+	}
+
+	int len = rz_str_unescape(str);
+	if (!rz_core_write_at(core, addr, (const ut8 *)str, len + 1)) {
+		RZ_LOG_ERROR("Could not write '%s' at %" PFMT64x "\n", s, addr);
+		free(str);
+		return false;
+	}
+	free(str);
+	return true;
+}
+
+/**
+ * \brief Transform a block of data at \p addr according to the operation \p op and the hexvalue \p hex
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Where the block of data to modify starts
+ * \param op Operation to perform on the block of data
+ * \param hex Optional hex string that may be required by the specific operation
+ * \param hexlen Optional length of the \p hex string. Must be present if \p hex is specified.
+ * \param buflen Used to return the length of the returned buffer
+ * \return The transformed buffer
+ */
+RZ_API RZ_OWN ut8 *rz_core_transform_op(RzCore *core, ut64 addr, RzCoreWriteOp op, ut8 *hex, int hexlen, int *buflen) {
+	rz_return_val_if_fail(core, NULL);
+	rz_return_val_if_fail(!hex || hexlen >= 0, NULL);
+	rz_return_val_if_fail(buflen, NULL);
+
+	switch (op) {
+	case RZ_CORE_WRITE_OP_ADD:
+	case RZ_CORE_WRITE_OP_SUB:
+	case RZ_CORE_WRITE_OP_DIV:
+	case RZ_CORE_WRITE_OP_MUL:
+	case RZ_CORE_WRITE_OP_AND:
+	case RZ_CORE_WRITE_OP_OR:
+	case RZ_CORE_WRITE_OP_XOR:
+	case RZ_CORE_WRITE_OP_SHIFT_LEFT:
+	case RZ_CORE_WRITE_OP_SHIFT_RIGHT:
+		rz_return_val_if_fail(hex && hexlen >= 0, NULL);
+		break;
+	default:
+		break;
+	}
+
+	ut8 *buf = RZ_NEWS(ut8, core->blocksize);
+	if (!buf) {
+		return NULL;
+	}
+
+	int len = rz_io_nread_at(core->io, addr, buf, core->blocksize);
+	if (len < 0) {
+		free(buf);
+		return NULL;
+	}
+
+	for (int i = 0, j = 0; i < len; i++, j = (j + 1) % (hexlen ? hexlen : 1)) {
+		ut16 tmp16;
+		ut32 tmp32;
+		ut64 tmp64;
+		switch (op) {
+		case RZ_CORE_WRITE_OP_BYTESWAP2:
+			if (i + 1 < len) {
+				tmp16 = rz_read_le16(buf + i);
+				rz_write_be16(buf + i, tmp16);
+				i++;
+			}
+			break;
+		case RZ_CORE_WRITE_OP_BYTESWAP4:
+			if (i + 3 < len) {
+				tmp32 = rz_read_le32(buf + i);
+				rz_write_be32(buf + i, tmp32);
+				i += 3;
+			}
+			break;
+		case RZ_CORE_WRITE_OP_BYTESWAP8:
+			if (i + 7 < len) {
+				tmp64 = rz_read_le64(buf + i);
+				rz_write_be64(buf + i, tmp64);
+				i += 7;
+			}
+			break;
+		case RZ_CORE_WRITE_OP_ADD:
+			buf[i] += hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_SUB:
+			buf[i] -= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_DIV:
+			buf[i] = hex[j] ? buf[i] / hex[j] : 0;
+			break;
+		case RZ_CORE_WRITE_OP_MUL:
+			buf[i] *= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_AND:
+			buf[i] &= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_OR:
+			buf[i] |= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_XOR:
+			buf[i] ^= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_SHIFT_LEFT:
+			buf[i] <<= hex[j];
+			break;
+		case RZ_CORE_WRITE_OP_SHIFT_RIGHT:
+			buf[i] >>= hex[j];
+			break;
+		default:
+			rz_warn_if_reached();
+			break;
+		}
+	}
+	*buflen = len;
+	return buf;
+}
+
+/**
+ * \brief Write a full block of data according to the operation \p op and the hexvalue \p hex
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Where the block of data to modify starts
+ * \param op Operation to perform on the block of data
+ * \param hex Optional hex string that may be required by the specific operation
+ * \param hexlen Optional length of the \p hex string. Must be present if \p hex is specified.
+ * \return true if the write operation succeeds, false otherwise
+ */
+RZ_API bool rz_core_write_block_op_at(RzCore *core, ut64 addr, RzCoreWriteOp op, ut8 *hex, int hexlen) {
+	int buflen;
+	ut8 *buf = rz_core_transform_op(core, addr, op, hex, hexlen, &buflen);
+	if (!buf) {
+		return false;
+	}
+
+	if (!rz_core_write_at(core, addr, buf, buflen)) {
+		RZ_LOG_ERROR("Could not write block operation at %" PFMT64x "\n", addr);
+		free(buf);
+		return false;
+	}
+
+	return true;
+}
+
+/**
+ * \brief Write a full block of data with a sequence
+ *
+ * Write a full block of data with a sequence of numbers starting from \p from
+ * up to \p to, with a step of \p step. The values are written as numbers of
+ * \p value_size bytes.
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Where the block of data to modify starts
+ * \param from From where to start the sequence of numbers
+ * \param to Where to stop in the sequence
+ * \param step Difference between two numbers in the sequence
+ * \param value_size Size of each number of the sequence, in bytes
+ * \return true if the write operation succeeds, false otherwise
+ */
+RZ_API bool rz_core_write_seq_at(RzCore *core, ut64 addr, ut64 from, ut64 to, ut64 step, int value_size) {
+	rz_return_val_if_fail(core, false);
+	rz_return_val_if_fail(value_size == 1 || value_size == 2 || value_size == 4 || value_size == 8, false);
+	ut64 max_val = (1ULL << (8 * value_size));
+	rz_return_val_if_fail(from < max_val, false);
+	rz_return_val_if_fail(to < max_val, false);
+
+	ut8 *buf = RZ_NEWS0(ut8, core->blocksize);
+	if (!buf) {
+		return false;
+	}
+
+	ut64 diff = to <= from ? max_val : to - from + 1;
+	ut64 p = from;
+	for (size_t i = 0; i < core->blocksize; i += value_size, p = (from + ((p + step - from) % diff)) % max_val) {
+		rz_write_ble(buf + i, p, rz_config_get_b(core->config, "cfg.bigendian"), value_size * 8);
+	}
+
+	if (!rz_core_write_at(core, addr, buf, core->blocksize)) {
+		RZ_LOG_ERROR("Could not write sequence [%" PFMT64d ", %" PFMT64d "] step=%" PFMT64d ",value_size=%d at %" PFMT64x "\n", from, to, step, value_size, addr);
+		free(buf);
+		return false;
+	}
+
+	free(buf);
+	return true;
+}
+
+/**
+ * \brief Copy \p len bytes from \p from to \p addr
+ *
+ * \param core Reference to RzCore instance
+ * \param addr Where the data should be copied to
+ * \param from Where the data should be read from
+ * \param len Number of bytes to copy, expected to not be negative
+ * \return true if the write operation succeeds, false otherwise
+ */
+RZ_API bool rz_core_write_duplicate_at(RzCore *core, ut64 addr, ut64 from, int len) {
+	rz_return_val_if_fail(core, false);
+	rz_return_val_if_fail(len >= 0, false);
+
+	bool res = false;
+	ut8 *data = RZ_NEWS(ut8, len);
+	if (!data) {
+		return false;
+	}
+
+	int n = rz_io_nread_at(core->io, from, data, len);
+	if (n < 0) {
+		RZ_LOG_ERROR("Cannot read data from %" PFMT64x ".\n", from);
+		goto err;
+	}
+	if (!rz_core_write_at(core, addr, data, n)) {
+		RZ_LOG_ERROR("Cannot write %d bytes to %" PFMT64x ".\n", n, addr);
+		goto err;
+	}
+	res = true;
+err:
+	free(data);
+	return res;
+}
