@@ -11,8 +11,6 @@
 #include <signal.h>
 #include <sys/types.h>
 
-#if DEBUGGER
-
 #include "native/drx.c" // x86 specific
 #include "rz_cons.h"
 
@@ -32,10 +30,9 @@ static int rz_debug_native_reg_write(RzDebug *dbg, int type, const ut8 *buf, int
 #endif
 
 #if __WINDOWS__
-//#include <windows.h>
 #include "native/windows/windows_debug.h"
 // TODO: Move these onto windows.h?
-RZ_API RzList *rz_w32_dbg_modules(RzDebug *); //ugly!
+RZ_API RzList *rz_w32_dbg_modules(RzDebug *); // ugly!
 RZ_API RzList *rz_w32_dbg_maps(RzDebug *);
 #define RZ_DEBUG_REG_T CONTEXT
 #ifdef NTSTATUS
@@ -86,8 +83,6 @@ RZ_API RzList *rz_w32_dbg_maps(RzDebug *);
 #define WAITPID_FLAGS 0
 #endif
 
-#endif /* IF DEBUGGER */
-
 /* begin of debugger code */
 #if DEBUGGER
 
@@ -102,7 +97,7 @@ static int rz_debug_handle_signals(RzDebug *dbg) {
 }
 #endif
 
-//this is temporal
+// this is temporal
 #if __APPLE__ || __linux__
 
 static char *rz_debug_native_reg_profile(RzDebug *dbg) {
@@ -129,9 +124,10 @@ static int rz_debug_native_step(RzDebug *dbg) {
 		return false;
 	}
 	return true;
-#else // linux
+#elif __linux__
 	return linux_step(dbg);
 #endif
+	return 0;
 }
 
 // return thread id
@@ -257,7 +253,7 @@ static int rz_debug_native_continue(RzDebug *dbg, int pid, int tid, int sig) {
 			rz_sys_perror("PTRACE_CONT");
 		}
 	}
-	//return ret >= 0 ? tid : false;
+	// return ret >= 0 ? tid : false;
 	return tid;
 #endif
 }
@@ -328,19 +324,19 @@ static RzDebugReasonType rz_debug_native_wait(RzDebug *dbg, int pid) {
 			bool autoload_pdb = dbg->corebind.cfggeti(core, "pdb.autoload");
 			if (autoload_pdb) {
 				PLIB_ITEM lib = r->lib;
-				dbg->corebind.cmdf(core, "\"o \\\"%s\\\" 0x%p\"", lib->Path, lib->BaseOfDll);
-				char *o_res = dbg->corebind.cmdstrf(core, "o~+%s", lib->Name);
-				int fd = atoi(o_res);
-				free(o_res);
-				if (fd) {
-					char *pdb_file = dbg->corebind.cmdstr(core, "i~dbg_file");
-					if (pdb_file && (rz_str_trim(pdb_file), *pdb_file)) {
-						if (!rz_file_exists(pdb_file + 9)) {
+				RzBinOptions opts = { 0 };
+				opts.obj_opts.baseaddr = (uintptr_t)lib->BaseOfDll;
+				RzBinFile *cur = rz_bin_cur(core->bin);
+				RzBinFile *bf = rz_bin_open(core->bin, lib->Path, &opts);
+				if (bf) {
+					const RzBinInfo *info = rz_bin_object_get_info(bf->o);
+					if (RZ_STR_ISNOTEMPTY(info->debug_file_name)) {
+						if (!rz_file_exists(info->debug_file_name)) {
 							dbg->corebind.cmdf(core, "idpd");
 						}
-						dbg->corebind.cmdf(core, "idp");
+						dbg->corebind.cmd(core, "idp");
 					}
-					dbg->corebind.cmdf(core, "o-%d", fd);
+					rz_bin_file_set_cur_binfile(core->bin, cur);
 				}
 			}
 		} else {
@@ -474,7 +470,7 @@ static RzDebugReasonType rz_debug_native_wait(RzDebug *dbg, int pid) {
 		return RZ_DEBUG_REASON_ERROR;
 	}
 
-	//eprintf ("rz_debug_native_wait: status=%d (0x%x) (return=%d)\n", status, status, ret);
+	// eprintf ("rz_debug_native_wait: status=%d (0x%x) (return=%d)\n", status, status, ret);
 
 #ifdef WAIT_ON_ALL_CHILDREN
 	if (ret != pid) {
@@ -619,7 +615,7 @@ static RzList *rz_debug_native_threads(RzDebug *dbg, int pid) {
 
 #if __sun || __NetBSD__ || __KFBSD__ || __OpenBSD__ || __DragonFly__
 
-//Function to read register from Linux, BSD, Android systems
+// Function to read register from Linux, BSD, Android systems
 static int bsd_reg_read(RzDebug *dbg, int type, ut8 *buf, int size) {
 	int showfpu = false;
 	int pid = dbg->pid;
@@ -740,7 +736,7 @@ static int rz_debug_native_reg_write(RzDebug *dbg, int type, const ut8 *buf, int
 #else
 		return bsd_reg_write(dbg, type, buf, size);
 #endif
-	} //else eprintf ("TODO: reg_write_non-gpr (%d)\n", type);
+	} // else eprintf ("TODO: reg_write_non-gpr (%d)\n", type);
 	return false;
 }
 
@@ -762,16 +758,24 @@ static int io_perms_to_prot(int io_perms) {
 
 #if __linux__
 static int sys_thp_mode(void) {
-	const char *thp = "/sys/kernel/mm/transparent_hugepage/enabled";
+	size_t i;
+	const char *thp[] = {
+		"/sys/kernel/mm/transparent_hugepage/enabled",
+		"/sys/kernel/mm/redhat_transparent_hugepage/enabled",
+	};
 	int ret = 0;
-	char *val = rz_file_slurp(thp, NULL);
-	if (val) {
-		if (strstr(val, "[madvise]")) {
-			ret = 1;
-		} else if (strstr(val, "[always]")) {
-			ret = 2;
+
+	for (i = 0; i < RZ_ARRAY_SIZE(thp); i++) {
+		char *val = rz_file_slurp(thp[i], NULL);
+		if (val) {
+			if (strstr(val, "[madvise]")) {
+				ret = 1;
+			} else if (strstr(val, "[always]")) {
+				ret = 2;
+			}
+			free(val);
+			break;
 		}
-		free(val);
 	}
 
 	return ret;
@@ -852,8 +856,8 @@ static RzDebugMap *linux_map_alloc(RzDebug *dbg, ut64 addr, int size, bool thp)
 	};
 
 	/* NOTE: Since kernel 2.4,  that  system  call  has  been  superseded  by
-       		 mmap2(2 and  nowadays  the  glibc  mmap()  wrapper  function invokes
-       		 mmap2(2)). If arch is x86_32 then usage mmap2() */
+		 mmap2(2 and  nowadays  the  glibc  mmap()  wrapper  function invokes
+		 mmap2(2)). If arch is x86_32 then usage mmap2() */
 	if (!strcmp(dbg->arch, "x86") && dbg->bits == 4) {
 		sc_name = "mmap2";
 	} else {
@@ -1006,7 +1010,7 @@ static RzList *rz_debug_native_map_get(RzDebug *dbg) {
 	char region[100], region2[100], perms[5];
 	FILE *fd;
 	if (dbg->pid == -1) {
-		//eprintf ("rz_debug_native_map_get: No selected pid (-1)\n");
+		// eprintf ("rz_debug_native_map_get: No selected pid (-1)\n");
 		return NULL;
 	}
 	/* prepend 0x prefix */
@@ -1305,7 +1309,7 @@ static int rz_debug_native_drx(RzDebug *dbg, int n, ut64 addr, int sz, int rwx,
 #else
 	eprintf("drx: Unsupported platform\n");
 #endif
-	return false;
+	return -1;
 }
 
 #if __linux__
@@ -1329,7 +1333,7 @@ static int rz_debug_native_drx(RzDebug *dbg, int n, ut64 addr, int sz, int rwx,
 
 static bool ll_arm32_hwbp_set(pid_t pid, ut64 addr, int size, int wp, int type) {
 	const unsigned byte_mask = (1 << size) - 1;
-	//const unsigned type = 2; // Write.
+	// const unsigned type = 2; // Write.
 	const unsigned enable = 1;
 	const unsigned control = byte_mask << 5 | type << 3 | enable;
 	(void)ptrace(PTRACE_SETHBPREGS, pid, -1, (void *)(size_t)addr);
@@ -1348,7 +1352,7 @@ static bool arm32_hwbp_del(RzDebug *dbg, RzBreakpoint *bp, RzBreakpointItem *b)
 
 #if (__arm64__ || __aarch64__) && defined(PTRACE_GETREGSET)
 // type = 2 = write
-//static volatile uint8_t var[96] __attribute__((__aligned__(32)));
+// static volatile uint8_t var[96] __attribute__((__aligned__(32)));
 
 static bool ll_arm64_hwbp_set(pid_t pid, ut64 _addr, int size, int wp, ut32 type) {
 	const volatile uint8_t *addr = (void *)(size_t)_addr; //&var[32 + wp];
@@ -1434,10 +1438,17 @@ static int rz_debug_native_bp(RzBreakpoint *bp, RzBreakpointItem *b, bool set) {
 		return set
 			? arm64_hwbp_add(dbg, bp, b)
 			: arm64_hwbp_del(dbg, bp, b);
+#elif __WINDOWS__
+		return set
+			? w32_hwbp_arm_add(dbg, bp, b)
+			: w32_hwbp_arm_del(dbg, bp, b);
 #elif __arm__ && __linux__
 		return set
 			? arm32_hwbp_add(dbg, bp, b)
 			: arm32_hwbp_del(dbg, bp, b);
+#elif __riscv
+		dbg = NULL;
+		return false;
 #endif
 	}
 	return false;
@@ -1473,8 +1484,8 @@ static RzList *xnu_desc_list(int pid) {
 			perror("too few bytes");
 			break;
 		}
-		//printf ("FD %d RWX %x ", i, vi.pfi.fi_openflags);
-		//printf ("PATH %s\n", vi.pvip.vip_path);
+		// printf ("FD %d RWX %x ", i, vi.pfi.fi_openflags);
+		// printf ("PATH %s\n", vi.pvip.vip_path);
 		desc = rz_debug_desc_new(i,
 			vi.pvip.vip_path,
 			xwrz_testwx(vi.pfi.fi_openflags),
@@ -1491,7 +1502,7 @@ static RzList *rz_debug_desc_native_list(int pid) {
 	return xnu_desc_list(pid);
 #elif __WINDOWS__
 	return w32_desc_list(pid);
-#elif __KFBSD__
+#elif __KFBSD__ || __NetBSD__
 	return bsd_desc_list(pid);
 #elif __linux__
 	return linux_desc_list(pid);
@@ -1517,7 +1528,7 @@ static int rz_debug_native_map_protect(RzDebug *dbg, ut64 addr, int size, int pe
 		"main@global(0) { sc(%p,%d,%d);\n"
 		":int3\n"
 		"}\n",
-		num, (void *)addr, size, io_perms_to_prot(perms));
+		num, (void *)(size_t)addr, size, io_perms_to_prot(perms));
 
 	rz_egg_reset(dbg->egg);
 	rz_egg_setup(dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
@@ -1602,7 +1613,11 @@ RzDebugPlugin rz_debug_plugin_native = {
 #elif __aarch64__ || __arm64__
 	.bits = RZ_SYS_BITS_16 | RZ_SYS_BITS_32 | RZ_SYS_BITS_64,
 	.arch = "arm",
+#if __WINDOWS__
+	.canstep = 0,
+#else
 	.canstep = 1,
+#endif
 #elif __arm__
 	.bits = RZ_SYS_BITS_16 | RZ_SYS_BITS_32 | RZ_SYS_BITS_64,
 	.arch = "arm",
